function [ Plotmatrix ] = GeneratePlotmatrix_TwoPass( Detmatrix )
%GENERATEPLOTMATRIX_TWOPASS
%
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% Author: John-Olav Bratteb, 2014                      %
% Norwegian University of Science and Technology (NTNU) %
% jobrattebo@gmail.com                                  %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%
%{
% Funksjonen sker i deteksjonsmatrisen MTD_tot etter deteksjonshoper med
% algoritmen "Two-pass" (Connected-component labeling) evaluerer plot
% tyngdepunkt, observasjonstidspunkt, doppler og strrelse, og genererer 
% plotmatrisen Plotmatrix fra dette.
% 
% 
% Input: 
% Detmatrix er en deteksjonsmatrise som inneholder verdier 0-255, hvor 0 er
% ingen deteksjon, og verdi ~= 0 gir mlt dopplerskift. 127 = 0 doppler
% 
% Output:
% Plotmatrix er en plotmatrise p flgende form:
% Plotmatrix =  [range1, azimuth1, obs.tid1, doppler1, strrelse1, rangecelle1, PRI1;
%                range2, azimuth2, obs.tid2, doppler2, strrelse2, rangecelle2, PRI2;
%                    ... 
%                rangeN, azimuthN, obs.tidN, dopplerN, strrelseN, rangecelleN, PRIN;]
% 
% range og azimuth er her polarkoordinatene til et plot( basert p en 
% deteksjonshops tyngdepunkt). Obs.tid er observasjonstidspunktet for et 
% plot. Doppler er et plots dopplerverdi, og strrelse er antallet
% deteksjoner i MTD_tot som ble sltt sammen til plottet.
%}

% tic
meters_per_range_cell = 1.25;
range_des_factor = 3;               % M settes lik det som settes i CNav
maxdet = 150000;                  % Maximum number of detections extracted
format long
Plotmatrix = zeros(1,7);
Detmatrix = double(Detmatrix);
[R, Az] = size(Detmatrix);
Plotlabels = zeros(R, Az);
detheapnr = 1;
Equiv_heapnr = double(zeros(2000,100));
Detheaps = [];
global filename
header = double(Detmatrix(1:8,:));

%% Linerinterpoler asimut-retningstempel for hvert PRI i Detmatrix for kt
% nyaktighet
asimut = 256*header(1,:)+header(2,:);
asimut_inkr = [1, abs(diff(asimut))]; % Detekterer pri'ene der asimut inkrementeres og 
% utvider med en 1'er i starten for  f med det aller frste asimutspranget.
nord_hopp = asimut_inkr > 1; %Finner pri for spranget fra 359 grader til null.
asimut_inkr(nord_hopp) = 1; % Setter ogs dette spranget til +1.
% asimut_inkr inneholder n 1'ere p alle vinkelsprang, ellers er den null.
asimut_inkr = logical(asimut_inkr); % Caster fra float til logical.
asimut_subset = asimut(asimut_inkr); %Alle registrerte vinkler i fila, men kun et stk av hver.
pri_hele = 1:1:length(asimut);
pri_subset = pri_hele(asimut_inkr); %pri'ene der asimut inkrementerer.
asimut_interpol = interp1(pri_subset, asimut_subset, pri_hele, 'linear');

%Quickfix for NaNs in the last elements of asimut_interpol
nans = find(isnan(asimut_interpol));
if numel(nans)>0
    last_diff = asimut_interpol(nans(1)-1)-asimut_interpol(nans(1)-2);
    for i = nans
        asimut_interpol(i) = asimut_interpol(i-1) + last_diff;
    end
end

% Roter Nord med et bias for tilpasning til Nord i et kart
asimut_interpol = mod(asimut_interpol - 4,360);

%% Sker deteksjonsmatrisen Detmatrix (bbs_mti) etter deteksjoner gir alle en label og finner ekvivalente detheapnr - Two-pass (first pass)
for i = 1:R
    for j = 1:Az
       if Detmatrix(i,j) ~= 0 && i > 5 && j > 1     % Funn av deteksjon utenfor range-celle 5
           N = Plotlabels(i-1,j);   % North-nabo ( nabo inn i range )
           W = Plotlabels(i,j-1);   % West-nabo ( nabo inn i azimut )
           if N ~= 0 || W ~= 0      % Hvis nord- eller vest-nabo har et detheapnr fra fr
               if N == W || N == 0 || W == 0        % Naboene er merket med samme detheapnr, eller bare en nabo har et detheapnr
                   Plotlabels(i,j) = max(N,W);
               elseif N ~= W && N ~= 0 && W ~= 0    % Naboene har forksjellig detheapnr
                   Equiv_heapnr(find(Equiv_heapnr(:,1)==0,1,'first'),1:2) = [N, W]; % detheapnummerne til N og W er ekvivalente
                   Plotlabels(i,j) = min(N,W);
               end
           else  % Begge naboer er 0 -> ny detheap oppdaget
               Plotlabels(i,j) = detheapnr;
               detheapnr = detheapnr + 1;
           end
           
       end
    end
end

clear N W

%% Slr sammen og klargjr liste med ekvivalente detheapnr
Equiv_heapnr = unique(Equiv_heapnr,'rows'); % fjern duplikater
Equiv_heapnr(Equiv_heapnr(:,1)==0,:)=[];    % fjern rader med nuller
for i = 1:size(Equiv_heapnr,1)              % sl sammen rekkene med minst ett felles element
    for j = 1:size(Equiv_heapnr,1)
        if i ~= j && Equiv_heapnr(i,1) ~= 0
            if max(ismember(Equiv_heapnr(i,Equiv_heapnr(i,:)>0),Equiv_heapnr(j,Equiv_heapnr(j,:)>0)))
                temp = union(Equiv_heapnr(i,1:2),Equiv_heapnr(j,1:2));
                Equiv_heapnr(i,1:size(temp,2)) = temp;
                clear temp
                Equiv_heapnr(j,:) = 0;          % rekke j blir til null-rekke
            end
        end
    end
end
Equiv_heapnr(Equiv_heapnr(:,1)==0,:)=[];    % fjern rekkene med nuller

%% Genererer matrisen Detheaps - Two-pass (second pass)
%Detheaps = [plotnr, det_r, det_az, det_doppler; 
%            plotnr, det_r2, det_az2, det_doppler2; 
%                          ...                     ]
Detheaps = zeros(maxdet,4);
heapcount = 1;
for i = 1:R
    for j = 1:Az
                if i == 372 && j == 3289 % Feilsking fil 14073
                    a = 1;
                end
        if Plotlabels(i,j) ~= 0 % for alle element merket med et plottnr
            % Hvis det finnes evivalente plottnr, sett elementets plotnr  
            % lik det laveste av nummerne som er ekvivalente til nvrende 
            % plotnr.
            equiv_row = mod(find(Equiv_heapnr==Plotlabels(i,j)),size(Equiv_heapnr,1));
            if numel(equiv_row) > 1                 % Spesialtilfeller med store hoper kan skape duplikater i equiv_heapnr
                if all(equiv_row == equiv_row(1))   % Finnes duplikater i rekken equiv_row i matrisen equiv_heapnr
                    equiv_row = equiv_row(1);       % Fjern duplikater
                end
                if numel(equiv_row) > 1             % Hvis fortsatt flere enn en rekke
                    temp = [];
                    for n = 1:numel(equiv_row)
                         temp = union(temp,Equiv_heapnr(equiv_row(n),:));
                         temp(temp==0)=[];
                    end
                    Equiv_heapnr(equiv_row(1),1:numel(temp)) = temp; % Rekke 1 lik union av rekkene
                    clear temp;
                    Equiv_heapnr(equiv_row(2:end),:)=[]; % Fjern de andre rekkene
                    equiv_row = equiv_row(1);
                end
            end
            if equiv_row % Hvis plotnr har ekvivalente nr
                % sett plotnr til det minste i rekken som ikke er null
%                 if i==579 && j == 3529
%                     a=1;
%                 end
                Plotlabels(i,j) = min( Equiv_heapnr(equiv_row,1:find(Equiv_heapnr(equiv_row,:)==0,1,'first')-1) ); % Sett plot label lik minste ekvivalente label
            end
            clear equiv_row;
            Detheaps(heapcount,:) = [Plotlabels(i,j), i, j, Detmatrix(i,j)];
			heapcount = heapcount + 1;
        end
    end
end
Detheaps = sortrows(Detheaps,1);        % Sorter slik at alle deteksjoner i samme detheap ligger etter hverandre i Detheaps
Detheaps = Detheaps(find(Detheaps(:,1)~=0,1,'first'):end,:);    % Fjerner alle rekker med null
% OBS merk at detheapnr n ikke er kontinuerlige. F.eks kan det vre at det
% ikke finnes noe plott nr 28, mens bde 27,29 og mange hundre flere finnes

%% Split detheaps into smaller heaps (plot-split)
    Detheaps(:,1) = Detheaps(:,1)+10;
counter = 1;
Max_DetCellDist = 20;
checked = boolean(zeros(size(Detheaps,1),1));
for i=1:size(Detheaps,1)
    if ~checked(i) && ( i == 1 || Detheaps(i,1) ~= Detheaps(i-1,1) ) % Hvis frste det i ny detheap
%         if i == 11977 % Stor gseflokk i fil 31288
%             a=1;
%         end
        heapnr = Detheaps(i,1);
        pr = Detheaps(i,2);
        paz = Detheaps(i,3);
        j = i+1;
        neighbors = zeros(1,Max_DetCellDist^2+20);
        neighbors(1) = i;
        while j<=size(Detheaps,1) && j - i < 400
%             if i > 11977 && j == 12026
%                 a=1;
%             end
            if Detheaps(j,1) == heapnr
                qr = Detheaps(j,2);
                qaz = Detheaps(j,3);
                dist = sqrt( (qr-pr)^2 + (qaz-paz)^2 );
                if dist < Max_DetCellDist
                    neighbors(j-i+1) = j;
                end

            end
            if Detheaps(j,1) > heapnr + 3 && Detheaps(j,1) > counter + 3
                break
            end
            j = j+1;
        end
        neighbors(neighbors==0)=[];
        Detheaps(neighbors,1) = counter;
        checked(neighbors)=1;
        counter = counter+1;

    end
end    
Detheaps = sortrows(Detheaps,1);        % Sorter slik at alle deteksjoner i samme detheap ligger etter hverandre i Detheaps
% Detheapnr er n kontinuerlig

%% Itererer hver detheap i Detheaps og ekstraherer plot til Plotmatrix
plotnr = 1;
for i = 1:size(Detheaps,1) 
    if i == 1 || Detheaps(i,1) ~= Detheaps(i-1,1) % Hvis frste det i ny detheap
        d = 0;
        center_r = 0;
        center_az = 0;
        plot_doppler = 0;
    end
    d = d + 1;  % Inkrementer antall deteksjoner i detheap
    center_r = center_r + Detheaps(i,2);    % Akkumulerer til snittberegninger
    center_az = center_az + Detheaps(i,3);
    plot_doppler = plot_doppler + Detheaps(i,4);

    if i == size(Detheaps,1) || Detheaps(i,1) ~= Detheaps(i+1,1) % Hvis siste det i detheap
        % Finn deteksjonshopens tyngepunkt (center_r,
        % center_az) ved gjennomsnitt av alle det i detheap
        center_r = round(center_r / d);
        center_az = round(center_az / d);
        % Center_r i meter
        center_r_m = center_r - 8;
        center_r_m = meters_per_range_cell*range_des_factor*center_r_m;
        
        % Hent azimut-komponent fra header til azimut-celle/PRI,
        center_az_deg = asimut_interpol(center_az);  % asimut i grader med 0/360 grader i retning Nord med kende grader med klokka
        center_az_deg = mod(90-center_az_deg,360);   % asimut i grader med 0/360 grader i retning Hyre/st med kende grader mot klokka

        % Doppler-verdi basert p gjennomsnitt av komponenter i detheap
        plot_doppler = plot_doppler / d;

        % Finn plotsize
        plotsize = d;
        
        % Finn Time of Observation (TO)
        TO = (str2double(filename(end-8:end-4))-1 )* 2.333; % TO for frste puls i scan'et. TO = 0 er frste puls i frste fil. T = 2.333 er hentet fra snittsberegninger for fil 14069-14155. 
    %              T = 2.333 s er usikker. For hele Case 1 (14069-14245) var
    %              snittrotasjonstiden 2.345 s, alts 12 ms variasjon her.
        for p = 1:center_az                              % Summere opp dt for alle pulser p frem til Plottets senter i azimut (center_az)
            TO = TO + (256*double(Detmatrix(3,p)) + double(Detmatrix(4,p)))*0.096*10^(-6);
        end

        % Oppdater Plotmatrix
        Plotmatrix(plotnr,:) = [center_r_m, center_az_deg, TO, plot_doppler, plotsize, center_r, center_az];

        % Oppdater for ny detheap
        plotnr = plotnr + 1;
    end
    
end
Plotmatrix = sortrows(Plotmatrix,3); % Sorter etter observasjonstidspunkt

%{
% %              Finn Time of Observation (TO)
%            TO = (str2double(filename(end-8:end-4))-1 )* 2.333; % TO for frste puls i scan'et. TO = 0 er frste puls i frste fil. T = 2.333 er hentet fra snittsberegninger for fil 14069-14155. 
% %              T = 2.333 s er usikker. For hele Case 1 (14069-14245) var
% %              snittrotasjonstiden 2.345 s, alts 12 ms variasjon her.
%            for p = 1:center_az                              % Summere opp dt for alle pulser p frem til Plottets senter i azimut (center_az)
%                TO = TO + (256*double(Detmatrix(3,p)) + double(Detmatrix(4,p)))*0.096*10^(-6);
%            end
% 
%            % Finn plot_doppler
% %                plot_doppler = Detmatrix(i,j); % Bruk doppler fra frste deteksjon funnet i detheap
% 
% %            plot_doppler = 0;                                % Doppler-verdi basert p gjennomsnitt av komponenter i detheap
% %            for k = 1:size(Detheap,1)                        % Sl sammen denne lkken med den for beregning av tyngdepunkt hvis den blir brukt.
% %                plot_doppler = plot_doppler + Detheap(k,3);
% %            end
% %            plot_doppler = plot_doppler / size(Detheap,1);
% 
%            % Finn plotsize
% %            plotsize = size(Detheap,1);
% 
%            % Oppdaterer Plotmatrisen med en plot
%            detheapnr = detheapnr + 1;                             % Inkrementerer Plotnr for plot
%            Plotmatrix(detheapnr,:) = [center_r, center_az, TO, plot_doppler, plotsize];
% 
% 
%        
%     end
% end
%}
%% Eval og rydding
%Plotting for evaluering
% figure()
% imagesc(Detmatrix)
% whitebg('white')
% % alpha(0.2)
% hold on
% scatter(Plotmatrix(:,7),Plotmatrix(:,6),200,'k','x','LineWidth',3)

clearvars -except Plotmatrix

% toc
end

